==> straw-0.4.0-sources.jar: org/zalando/straw/Straw.java <==
package org.zalando.straw;

import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


public class Straw {

    public static final int SECOND = 1000;
    public static final String OFFSET_BEGIN = "begin";

    public static final class HttpException extends Exception {
        public HttpException(String message) {
            super(message);
        }
    }

    static final class Cursor {

        static List<String> parse(String batch) throws Exception {
            List<String> result = new ArrayList();
            Scanner scanner = new Scanner(batch);
            String match;
            while ((match = scanner.findInLine("\"\\w+\"")) != null) {
                match = match.substring(1, match.length() - 1); //remove doublequotes
                result.add(match);
            }
            return result;
        }

        static Cursor extract(String batch) throws Exception {
            List<String> parsed = parse(batch);
            if ("cursor".equals(parsed.get(0))) {
                 if ("partition".equals(parsed.get(1)) && "offset".equals(parsed.get(3))) {
                    return new Cursor(parsed.get(2), parsed.get(4));
                 } else if ("partition".equals(parsed.get(3)) && "offset".equals(parsed.get(1))) {
                     return new Cursor(parsed.get(4), parsed.get(2));
                 } else {
                     throw new Exception("malformed batch: " + batch);
                 }
            } else {
                throw new Exception("malformed batch: " + batch);
            }
        }

        final String partition;
        final String offset;

        Cursor(String partition, String offset) {
            this.partition = partition;
            this.offset = offset;
        }

        @Override public String toString() {
            return String.format("{\"partition\":\"%d\",\"offset\":\"%s\"}", partition, offset);
        }
    }

    private final ExecutorService _executor = Executors.newSingleThreadExecutor();
    private final URL _url;
    private final Map<String, String> _cursors;
    private final boolean _allPartitions;

    public Straw(URL url, Map<String, String> cursors) {
        _url = url;
        _cursors = new HashMap(cursors);
        _allPartitions = cursors.isEmpty();
    }

    public void start() {
        _executor.submit(() -> { while (true) fetchStream(); });
    }

    protected String loadToken() throws Exception {
        return System.getenv("TOKEN");
    }

    protected void handleEvents(String json) throws Exception {
        logDebug("handleEvents: " + json);
    }

    protected void logDebug(String message) { System.out.println("DEBUG: " + message); }

    protected void logInfo(String message) { System.out.println("INFO: " + message); }

    protected void logError(Exception e) { e.printStackTrace(System.err); }

    private void fetchStream() {
        logInfo("fetchStream: " + (_allPartitions ? "all partitions (END)" : cursorString()));
        try {
            SSLSocket socket = (SSLSocket) SSLSocketFactory.getDefault().createSocket(_url.getHost(), port());
            try {
                socket.setSoTimeout(60 * SECOND);
                sendRequest(socket);
                BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream(), "UTF-8"));
                skipHeaders(in);
                // simple state machine to read chunked encoding. each line can be SIZE, DATA or EMPTY.
                // a single nakadi batch can span multiple chunks, for example:
                // SIZE DATA SIZE DATA SIZE DATA EMPTY
                StringBuilder batch = new StringBuilder();
                boolean isData = false;
                String line;
                while ((line = in.readLine()) != null) {
                    if (isData) {
                        batch.append(line);
                        isData = false;
                    } else {
                        if (line.isEmpty()) {
                            handleBatch(batch.toString());
                            batch.setLength(0);
                        } else {
                            isData = true;
                        }
                    }
                }
            } finally {
                socket.close();
            }
        } catch (Exception e) {
            logError(e);
            tryToSleep(2 * SECOND);
        }
    }

    private void handleBatch(String batch) throws Exception {
        if (!batch.isEmpty()) {
            Cursor cursor = Cursor.extract(batch);
            handleEvents(batch);
            // no exception, so we can update _cursors
            if (offsetGreaterThan(cursor.offset, _cursors.getOrDefault(cursor.partition, OFFSET_BEGIN))) {
                _cursors.put(cursor.partition, cursor.offset);
            } 
        }
    }

    private void sendRequest(SSLSocket socket) throws Exception {
        socket.startHandshake();
        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())));
        out.println("GET " + requestPath() + " HTTP/1.1");
        out.println("Host: " + _url.getHost());
        out.println("User-Agent: straw");
        if (!_allPartitions) out.println("X-Nakadi-Cursors: " + cursorString());
        String token = loadToken();
        if (token != null) out.println("Authorization: Bearer " + token.trim());
        out.println();
        out.flush();
    }

    private int port() {
        return _url.getPort() == -1 ? 443 : _url.getPort();
    }

    private String requestPath() {
        return _url.getQuery() == null ? _url.getPath() : _url.getPath() + "?" + _url.getQuery();
    }

    private String cursorString() {
        List<Cursor> result = new ArrayList();
        for (String partition : _cursors.keySet()) {
            result.add(new Cursor(partition, _cursors.get(partition)));
        }
        return Arrays.toString(result.toArray());
    }

    private static void skipHeaders(BufferedReader out) throws IOException, HttpException {
        String line, status = null;
        while ((line = out.readLine()) != null) {
            if (status == null) {
                status = line.split("\\s", 2)[1]; // skip protocol part
                if (!status.startsWith("200")) throw new HttpException(status);
            } else if (line.trim().isEmpty()) {
                 break;
            }
        }
    }

    private static boolean offsetGreaterThan(String a, String b) {
        return OFFSET_BEGIN.equals(b) ? true : a.compareTo(b) > 0;
    }

    private static void tryToSleep(int millis) {
        try { Thread.sleep(millis); } catch (InterruptedException ignored) {}
    }
}

==> straw-0.4.1-sources.jar: org/zalando/straw/Straw.java <==
package org.zalando.straw;

import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


public class Straw {

    public static final int SECOND = 1000;
    public static final String OFFSET_BEGIN = "begin";

    public static final class HttpException extends Exception {
        public HttpException(String message) {
            super(message);
        }
    }

    static final class Cursor {

        static List<String> parse(String batch) throws Exception {
            List<String> result = new ArrayList();
            Scanner scanner = new Scanner(batch);
            String match;
            while ((match = scanner.findInLine("\"\\w+\"")) != null) {
                match = match.substring(1, match.length() - 1); //remove doublequotes
                result.add(match);
            }
            return result;
        }

        static Cursor extract(String batch) throws Exception {
            List<String> parsed = parse(batch);
            if ("cursor".equals(parsed.get(0))) {
                 if ("partition".equals(parsed.get(1)) && "offset".equals(parsed.get(3))) {
                    return new Cursor(parsed.get(2), parsed.get(4));
                 } else if ("partition".equals(parsed.get(3)) && "offset".equals(parsed.get(1))) {
                     return new Cursor(parsed.get(4), parsed.get(2));
                 } else {
                     throw new Exception("malformed batch: " + batch);
                 }
            } else {
                throw new Exception("malformed batch: " + batch);
            }
        }

        final String partition;
        final String offset;

        Cursor(String partition, String offset) {
            this.partition = partition;
            this.offset = offset;
        }

        @Override public String toString() {
            return String.format("{\"partition\":\"%s\",\"offset\":\"%s\"}", partition, offset);
        }
    }

    private final ExecutorService _executor = Executors.newSingleThreadExecutor();
    private final URL _url;
    private final Map<String, String> _cursors;
    private final boolean _allPartitions;

    public Straw(URL url, Map<String, String> cursors) {
        _url = url;
        _cursors = new HashMap(cursors);
        _allPartitions = cursors.isEmpty();
    }

    public void start() {
        _executor.submit(() -> { while (true) fetchStream(); });
    }

    protected String loadToken() throws Exception {
        return System.getenv("TOKEN");
    }

    protected void handleEvents(String json) throws Exception {
        logDebug("handleEvents: " + json);
    }

    protected void logDebug(String message) { System.out.println("DEBUG: " + message); }

    protected void logInfo(String message) { System.out.println("INFO: " + message); }

    protected void logError(Exception e) { e.printStackTrace(System.err); }

    private void fetchStream() {
        logInfo("fetchStream: " + (_allPartitions ? "all partitions (END)" : cursorString()));
        try {
            SSLSocket socket = (SSLSocket) SSLSocketFactory.getDefault().createSocket(_url.getHost(), port());
            try {
                socket.setSoTimeout(60 * SECOND);
                sendRequest(socket);
                BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream(), "UTF-8"));
                skipHeaders(in);
                // simple state machine to read chunked encoding. each line can be SIZE, DATA or EMPTY.
                // a single nakadi batch can span multiple chunks, for example:
                // SIZE DATA SIZE DATA SIZE DATA EMPTY
                StringBuilder batch = new StringBuilder();
                boolean isData = false;
                String line;
                while ((line = in.readLine()) != null) {
                    if (isData) {
                        batch.append(line);
                        isData = false;
                    } else {
                        if (line.isEmpty()) {
                            handleBatch(batch.toString());
                            batch.setLength(0);
                        } else {
                            isData = true;
                        }
                    }
                }
            } finally {
                socket.close();
            }
        } catch (Exception e) {
            logError(e);
            tryToSleep(2 * SECOND);
        }
    }

    private void handleBatch(String batch) throws Exception {
        if (!batch.isEmpty()) {
            Cursor cursor = Cursor.extract(batch);
            handleEvents(batch);
            // no exception, so we can update _cursors
            if (offsetGreaterThan(cursor.offset, _cursors.getOrDefault(cursor.partition, OFFSET_BEGIN))) {
                _cursors.put(cursor.partition, cursor.offset);
            } 
        }
    }

    private void sendRequest(SSLSocket socket) throws Exception {
        socket.startHandshake();
        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())));
        out.println("GET " + requestPath() + " HTTP/1.1");
        out.println("Host: " + _url.getHost());
        out.println("User-Agent: straw");
        if (!_allPartitions) out.println("X-Nakadi-Cursors: " + cursorString());
        String token = loadToken();
        if (token != null) out.println("Authorization: Bearer " + token.trim());
        out.println();
        out.flush();
    }

    private int port() {
        return _url.getPort() == -1 ? 443 : _url.getPort();
    }

    private String requestPath() {
        return _url.getQuery() == null ? _url.getPath() : _url.getPath() + "?" + _url.getQuery();
    }

    private String cursorString() {
        List<Cursor> result = new ArrayList();
        for (String partition : _cursors.keySet()) {
            result.add(new Cursor(partition, _cursors.get(partition)));
        }
        return Arrays.toString(result.toArray());
    }

    private static void skipHeaders(BufferedReader out) throws IOException, HttpException {
        String line, status = null;
        while ((line = out.readLine()) != null) {
            if (status == null) {
                status = line.split("\\s", 2)[1]; // skip protocol part
                if (!status.startsWith("200")) throw new HttpException(status);
            } else if (line.trim().isEmpty()) {
                 break;
            }
        }
    }

    private static boolean offsetGreaterThan(String a, String b) {
        return OFFSET_BEGIN.equals(b) ? true : a.compareTo(b) > 0;
    }

    private static void tryToSleep(int millis) {
        try { Thread.sleep(millis); } catch (InterruptedException ignored) {}
    }
}
